<?php


namespace Sample;

use Analog\Handler\File;
use Analog\Logger;
use ESET\SecureAuthentication\SDK\CredentialTypes;
use ESET\SecureAuthentication\SDK\Exceptions\UserLockedException;
use ESET\SecureAuthentication\SDK\SdkLogManager;
use ESET\SecureAuthentication\SDK\IUserStorage;
use ESET\SecureAuthentication\SDK\TwoFactorAuthenticator;
use ESET\SecureAuthentication\SDK\TwoFactorConfiguration;


class Program
{
    const API_KEY = "";
    const API_SECRET = "";

    const TEST_USERNAME = "testuser";

    /** @var  TwoFactorAuthenticator */
    private $authenticator;

    /** @var  string Root directory of app, used for log & datastore files */
    private $rootDir;

    public function __construct($rootDir)
    {
        $this->rootDir = $rootDir;
        $this->authenticator = $this->getTwoFactorAuthenticator(self::API_KEY, self::API_SECRET);
    }

    /**
     * Create a TwoFactorAuthenticator object that is configuration with our choices
     * @param $apiKey
     * @param $apiSecret
     * @return TwoFactorAuthenticator
     */
    private function getTwoFactorAuthenticator($apiKey, $apiSecret)
    {
        // Turn on logging with Analog (or any log framework conforming to PSR-3)
        $logger = new Logger();
        $logger->handler(File::init($this->rootDir . '/log.txt'));
        SdkLogManager::setLogger($logger);

        /** @var  IUserStorage User data storage implementation */
        $userStorage = new CustomUserStorage($this->rootDir . '/datastore.txt');

        // Create a configuration object
        // All SDK settings may be changed here
        $configuration = new TwoFactorConfiguration($apiKey, $apiSecret, $userStorage);
        $configuration->setTokenName('PHP SDK Token');

        $authenticator = new TwoFactorAuthenticator($configuration);

        return $authenticator;
    }


    public function run()
    {
        $this->provisionUserTextMessage();

        // Always run these functions together
        $this->preAuthenticate();
        $this->authenticateUser();

        Console::writeLine();
        Console::writeLine("We now transition the user to a mobile app...");

        // unlock the user in case operator (that's you!) has entered too many incorrect OTPs
        $this->unlockUser();
        $this->provisionUserMobileApp();

        Console::writeLine();
        Console::writeLine("Press <ENTER> when app is installed.");
        Console::readLine();

        Console::writeLine("The user may now use a text message or mobile app OTP.");
        Console::writeLine("Once a mobile app OTP has been used, no more text message OTPs will be sent.");

        do {
            // preAuthenticate & authenticate work regardless of the OTP type in use
            $this->preAuthenticate();
            $this->authenticateUser();

            // unlock the user in case operator (that's you!) has entered too many incorrect OTPs
            $this->unlockUser();

        } while ($this->authenticator->getUser(self::TEST_USERNAME)->getCredentialType() != CredentialTypes::APP_HOTP_CREDENTIAL);

        Console::writeLine();
        Console::writeLine("The user has been transitioned to a mobile app.");
        Console::writeLine("The next preAuthenticate call will not send a text message OTP.");

        // preAuthenticate & authenticate work regardless of the OTP type in use
        $this->preAuthenticate();
        $this->authenticateUser();

    }

    private function provisionUserTextMessage()
    {
        Console::write(
            "Enter mobile number for user in international format" . PHP_EOL .
            " (e.g. '42612345678', where '4' is the country code and '26' is the area code):"
        );

        $mobileNumber = Console::readLine();

        $twoFactorUser = $this->authenticator->createUser(self::TEST_USERNAME, $mobileNumber);
        $twoFactorUser->provisionTextMessage();

        // NOTE: Must always call save after working with a user!
        $twoFactorUser->save();

        Console::writeLine("The user has been provisioned for text message OTPs.");

    }

    private function provisionUserMobileApp()
    {
        $user = $this->authenticator->getUser(self::TEST_USERNAME);

        $provisionResult = $user->provisionMobileApp();
        $user->sendMobileAppProvisioningTextMessage($provisionResult);
        $user->save();

        Console::writeLine("The user has been provisioned for a mobile app and will receive a text message link shortly.");

    }

    private function unlockUser()
    {
        $user = $this->authenticator->getUser(self::TEST_USERNAME);
        if ($user->getIsLocked()) {
            $user->unlock();
        }
        $user->save();
    }

    private function preAuthenticate()
    {
        Console::writeLine();
        Console::writeLine("     Calling preAuthenticate for user...");

        $result = $this->authenticator->preAuthenticate(self::TEST_USERNAME);

        Console::writeLine("     Expected credential: " . CredentialTypes::name($result->getExpectedCredential()));
    }

    private function authenticateUser()
    {
        // Keep prompting for OTPs until success or user becomes locked
        while (true) {
            Console::writeLine();
            Console::write("     Enter OTP: ");
            $otp = Console::readLine();

            try {
                $result = $this->authenticator->authenticate(self::TEST_USERNAME, $otp);
                if ($result->getIsAuthenticated()) {
                    Console::writeLine("     OTP has been authenticated.");
                    break;
                }
                Console::writeLine("     OTP has not been authenticated.");

            } catch (UserLockedException $ex) {
                Console::writeLine("     OTP cannot be authenticated - user has been locked.");
                break;
            }
        }
    }
}
